--[[ This program is freeware; you may give it away and/or change it
under the GNU General Public License.  However, there is no
warrenty; not even an implied one.  When I say warrenty, you
say, "None!"  Also of note is that only the source code is
licensed under the GNU General Public License.  The art cannot
be edited, changed, or included with other programs.

The other software that this program uses is under a copyright
from the people whom wrote it, mostly being a variation on the
zlib license or BSD license.
For more information on the license, check the COPYING file
included with this program, or look on the GNU website,
www.gnu.org ]]

--[[
  What's contained in this file:
  * Collision detection for the enemies, the hero and their bullets.
]]

lhc = require "libs/hardoncollider"
lhcShapes = require "libs/hardoncollider.shapes"

Collision = {}

local gf = love.graphics
local __abs = math.abs

-- In the constructor give the objects of the bullet table, main player table,
-- and enemies table as parameters.
function Collision.new(bullets, hero, badGuys)

  local arbiter = {}
  
  function arbiter.startCollider()
    lhc.init(100, arbiter.startcolliding, arbiter.while_colliding)
  end
  
  function arbiter.moveBoundary(shape, x, y)
    if not shape then error("Please provide shape object.") end
    shape:moveTo(x,y) -- Move the shape from one point to another.
  end
  
--   function arbiter.checkForEnemyCollision(hero, badGuys)
--   if not hero then error("Please provide \"hero\" object.") end
--   if not badGuys then error("Please provide \"badGuys\" object.") end
--   for i,b in pairs(badGuys.enemy)  do
--     if (__abs((hero.x+hero.radius)-(badGuys.enemy[i].x+badGuys.enemy[i].radius)) < hero.radius and __abs((hero.y+hero.radius)-(badGuys.enemy[i].y+badGuys.enemy[i].radius)) < hero.radius) then
--       if not hero.hasBeenHit then hero.hasBeenHit = true end
--     end
--   end
--  end

 -- Bullet detection
  -- Enemy's bullets
--  function arbiter.checkForEnemyBulletCollision(bullets, hero)
--   for i,b in pairs(bullets) do
--    if b.who == "hamster" then
    -- __squareRoot((hero.x-b.x)^2+(hero.y-b.y)^2) < hero.radius
--      if (__abs((hero.x+hero.radius)-b.x) < hero.radius and __abs((hero.y+hero.radius)-b.y) < hero.radius) then
--       if not hero.hasBeenHit then hero.hasBeenHit = true end
--      end
--    end
--   end

  function arbiter.updateCollider(x)
   if not x then x = 30 end
  -- Auto update the Hardon Collider.
   lhc.setAutoUpdate(x)
  end
  
  -- This function is for collision detection, specifically circles that follow the enemy
  -- when they move, and raise flags when they collide.
  function arbiter.loadEnemyShapes(badGuys)
    if not badGuys then badGuys = badGuys end --This doesn't make sense, does it?
    badGuys.enemy[1].buzzard1.circle = lhc.addCircle(badGuys.enemy[1].buzzard1.x, badGuys.enemy[1].buzzard1.y+64, 64)
    badGuys.enemy[1].buzzard1.circle.hasBeenHit = false
    
    -- Add 32 to the coordinates to center the boundary.
    badGuys.enemy[2].buzzard1.circle = lhc.addCircle(badGuys.enemy[2].buzzard1.x+64, badGuys.enemy[2].buzzard1.y+64, 20)
    badGuys.enemy[2].buzzard1.circle.hasBeenHit = false
    badGuys.enemy[2].buzzard2.circle = lhc.addCircle(badGuys.enemy[2].buzzard2.x+64, badGuys.enemy[2].buzzard2.y+64, 20)
    badGuys.enemy[2].buzzard2.circle.hasBeenHit = false
    badGuys.enemy[2].buzzard3.circle = lhc.addCircle(badGuys.enemy[2].buzzard3.x+64, badGuys.enemy[2].buzzard3.y+64, 20)
    badGuys.enemy[2].buzzard3.circle.hasBeenHit = false
    badGuys.enemy[2].buzzard4.circle = lhc.addCircle(badGuys.enemy[2].buzzard4.x+64, badGuys.enemy[2].buzzard4.y+64, 20)
    badGuys.enemy[2].buzzard4.circle.hasBeenHit = false
    
    badGuys.enemy[3].buzzard1.circle = lhc.addCircle((SCREENWIDTH-100)+48, -44+48, 32)
    badGuys.enemy[3].buzzard1.circle.hasBeenHit = false
    badGuys.enemy[3].buzzard2.circle = lhc.addCircle(40+48, -44+48, 32)
    badGuys.enemy[3].buzzard2.circle.hasBeenHit = false
    
    badGuys.enemy[4].buzzard1.circle = lhc.addCircle(-158+32, 158+32, 32)
    badGuys.enemy[4].buzzard1.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard2.circle = lhc.addCircle(-94+32, 94+32, 32)
    badGuys.enemy[4].buzzard2.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard3.circle = lhc.addCircle(-30+32, 30+32, 32)
    badGuys.enemy[4].buzzard3.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard4.circle = lhc.addCircle(94+32, -94+32, 32)
    badGuys.enemy[4].buzzard4.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard5.circle = lhc.addCircle(158+32, -158+32, 32)
    badGuys.enemy[4].buzzard5.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard6.circle = lhc.addCircle( (SCREENWIDTH+158)+32, 158+32, 32)
    badGuys.enemy[4].buzzard6.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard7.circle = lhc.addCircle( (SCREENWIDTH+94), 94+32, 32)
    badGuys.enemy[4].buzzard7.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard8.circle = lhc.addCircle( (SCREENWIDTH+30), 30+32, 32)
    badGuys.enemy[4].buzzard8.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard9.circle = lhc.addCircle( (SCREENWIDTH-94), 94+32, 32)
    badGuys.enemy[4].buzzard9.circle.hasBeenHit = false
    badGuys.enemy[4].buzzard10.circle = lhc.addCircle( (SCREENWIDTH-158), -158+32, 32)
    badGuys.enemy[4].buzzard10.circle.hasBeenHit = false
    print("Enemy ship boundaries loaded.")
  end
  

 function arbiter.addEnemiesToCollisionGroup(badGuys)
  if not badguys then badguys = badguys end -- Doesn't this sound complicated.
    lhc.addToGroup("enemy", badGuys.enemy[1].buzzard1.circle)
    
    lhc.addToGroup("enemy", badGuys.enemy[2].buzzard1.circle)
    lhc.addToGroup("enemy", badGuys.enemy[2].buzzard2.circle)
    lhc.addToGroup("enemy", badGuys.enemy[2].buzzard3.circle)
    lhc.addToGroup("enemy", badGuys.enemy[2].buzzard4.circle)
    
    lhc.addToGroup("enemy", badGuys.enemy[3].buzzard1.circle)
    lhc.addToGroup("enemy", badGuys.enemy[3].buzzard2.circle)
    
    for i=1,10 do
      lhc.addToGroup("enemy", badGuys.enemy[4]["buzzard"..i].circle)
    end
    print("Enemies added to the 'enemy' group.")
 end
 

   -- Code aided by the users thelinx and vrid.
  function arbiter.start_colliding(changeInTime, thing1, thing2)
    if not changeInTime then changeInTime =  love.timer.getDelta() end
    --  and thing2._groups.bullets.who == "enemy" and thing1._groups.bullets.who == "enemy"
    if (thing1._groups.enemy and thing2._groups.bullets) or (thing1._groups.bullets and thing2._groups.enemy) then
      local enemy = thing1._groups.enemy and thing1 or thing2  -- Got to love ternary operations.
      if not enemy.hasBeenHit then
	enemy.hasBeenHit = true
        print("Enemy has started colliding.")
      end
    end
  end
  
  function arbiter.while_colliding(changeInTime, thing1, thing2)
    if not changeInTime then changeInTime = love.timer.getDelta() end
    -- Check which parameter has which group.
    if (thing1._groups.enemy and thing2._groups.bullets) then
      local enemy = thing1  -- Code fixed by vrid (hey, he's the guy who wrote the darn library in the first place).
      local bullets = thing2
      -- Check to see if the bullet is meant for the enemy.
      if bullets.who == "enemy" then
	enemy.hasBeenHit = true
      end
      
    elseif (thing1._groups.bullets and thing2._groups.enemy) then
      local bullets = thing1
      local enemy = thing2
      if bullets.who == "enemy" then
	enemy.hasBeenHit = true
      end
    end
  end
  
 return arbiter
end