--[[ This program is freeware; you may give it away and/or change it
under the GNU General Public License.  However, there is no
warrenty; not even an implied one.  When I say warrenty, you
say, "None!"  Also of note is that only the source code is
licensed under the GNU General Public License.  The art cannot
be edited, changed, or included with other programs.

The other software that this program uses is under a copyright
from the people whom wrote it, mostly being a variation on the
zlib license or BSD license.
For more information on the license, check the COPYING file
included with this program, or look on the GNU website,
www.gnu.org ]]

-- ### Maybe the tables for the enemies should be in another file? ### 

--[[
What's contained in this file:
  * The data for the enemies.
  * The functions for moving and animating the enemies.
  * The functions for animating enemy bullets.
  * The functions for tracking enemy "health".
]]

lhc = require "libs/hardoncollider"
lhcShapes = require "libs/hardoncollider.shapes"
collision = "libs/collision.lua"

Enemies = {}

local gf = love.graphics
local SCREENHEIGHT = gf.getHeight()
local SCREENWIDTH = gf.getWidth()

-- These variables are used to maintain the pace of
-- the enemy waves.
local wave1 = 3
local wave2 = 7.2
local wave3 = 11.3
local wave4 = 17.1

local PI_DIVIDED_BY_180 = 0.017453292519943  -- This is just an approximation,
				             -- but it's close enough for me.

function Enemies.new()

  -- Put them all in here.
  local swarm = {}

  -- Create a table for enemies.
  swarm.enemy = {}
  
  -- Set some attributes (at least I hope these are some attributes).
  elaspedTime = 0
  local tableOfImages = {}

  -- Use this function to set the time from the last refresh so that it can be used in the
  -- draw_enemies() function.
  function swarm.setElapsedTime(totalTime)
    elaspedTime = totalTime
  end
  
  -- This is for sending out each wave after a certain amount of time has
  -- gone past.
  function swarm.checkTimePast()
    if elaspedTime >= wave1 then
      swarm.enemy[1].readyToMove = 1
    end
    
    if elaspedTime >= wave2 then
      swarm.enemy[2].readyToMove = 1
    end
    
    if elaspedTime >= wave3 then
      swarm.enemy[3].readyToMove = 1
    end
    
    if elaspedTime >= wave4 then
      swarm.enemy[4].readyToMove = 1
    end
  end
  
  -- This is for checking to see if the enemy has gone off the screen.  Returns true if they are.
  -- Code aided by the user Robin.
--   function swarm.isOffScreen(object, radius)
--    if not object then error("Please enter in enemy object to the function.") end
--    if not radius then radius = 32 end
--    return object.x < -radius or object.y < -radius or object.x > SCREENHEIGHT or object.y > SCREENWIDTH
--   end
  -- ### Note: should this be placed in the collisions.lua file? ###
  -- Code suggested by the user Lafolie
  function swarm.isOffScreen(enemy, image)
    if enemy.x - image:getWidth() > 0 and enemy.x + image:getWidth() < SCREENWIDTH  and enemy.y - image:getHeight() > 0  and enemy.y + image:getHeight() < SCREENHEIGHT then
      return false -- It's still on the screen.
    else
      return true -- It's off the screen.
    end
  end


  -- Enemy table #1.
  swarm.enemy[1] = { radius = 64,
    buzzard1 = {
    x = 300, y= -128, speed = 125, timeshot = 0, timeBetween = 0.4, hasBeenHit = false, isDead = false
    }, fireWhenReady = true, isItReadyToMove = nil, circle = nil, image = "images/enemy1.png",
  zigzag = function(changeInTime)
   swarm.enemy[1].buzzard1.y = swarm.enemy[1].buzzard1.y + (swarm.enemy[1].buzzard1.speed * changeInTime)
   -- Good ol' Sine wave.
   swarm.enemy[1].buzzard1.x = swarm.enemy[1].buzzard1.x + 2*math.sin(swarm.enemy[1].buzzard1.y * PI_DIVIDED_BY_180)
   if swarm.enemy[1].buzzard1.y >= SCREENHEIGHT then  -- See if this enemy has left the screen.
     swarm.enemy[1].buzzard1.fireWhenReady = false
   end
  end,
  -- Code for firing off this enemy's weapon.
  fireMainGun = function(x_coor, y_coor, changeInTime)
    swarm.enemy[1].buzzard1.timeshot = swarm.enemy[1].buzzard1.timeshot - changeInTime
    if swarm.enemy[1].buzzard1.timeshot <= 0 then
     armory.enemyWeapon[1].fireWeapon(x_coor, y_coor)
     -- Play sound when the enemy fires off their bullets.
     TEsound.play("sounds/enemy_shooting_sound_1.ogg")
     -- Slow 'er down.
     swarm.enemy[1].buzzard1.timeshot = swarm.enemy[1].buzzard1.timeBetween
    end
  end,
   -- This is for moving this particular enemy in sync with the proper time.
   moveBuzzard = function()
     -- This is for when the enemy dies.
     if not swarm.enemy[1].buzzard1.circle.hasBeenHit then
       gf.draw(enemy_image, swarm.enemy[1].buzzard1.x, swarm.enemy[1].buzzard1.y)
       collision.moveBoundary(swarm.enemy[1].buzzard1.circle, swarm.enemy[1].buzzard1.x, swarm.enemy[1].buzzard1.y)
     end
   end
  }

  -- Enemy table #2.
  -- Sheesh, enemy tracking and enemy movement is tough.
   swarm.enemy[2] = { radius = 12, speed = 200, timeshot = 0, timeBetween = 0.8,
    isOffScreen = false, readyToMove = nil, image = "images/avatar37.jpg",
    buzzard1 = {  x = -64, y = 10, hasBeenHit = false, circle = nil }, -- Coming in from the left.
    buzzard2 = {  x = -64, y = 85, hasBeenHit = false, circle = nil }, -- Coming in from the left.
    buzzard3 = {  x = SCREENWIDTH, y = 10, hasBeenHit = false, circle = nil }, -- Coming in from the right.
    buzzard4 = {  x = SCREENWIDTH, y = 85, hasBeenHit = false, circle = nil }, -- Coming in from the right.
    -- Attack pattern of the second wave of enemies.
    pincerAttack = function(changeInTime)
    -- Let's make this a little easier.
     local buzzard_1 = swarm.enemy[2].buzzard1
     local buzzard_2 = swarm.enemy[2].buzzard2
     local buzzard_3 = swarm.enemy[2].buzzard3
     local buzzard_4 = swarm.enemy[2].buzzard4
     local slowerSpeed = 0.125
     -- Move them down the screen slowly.
     if buzzard_1.x <= 110 and buzzard_2.x <= 110 and buzzard_3.x >= SCREENWIDTH-74 and buzzard_4.x >= SCREENWIDTH-74 then
     -- The "x" component.
      swarm.enemy[2].buzzard1.x = swarm.enemy[2].buzzard1.x + (swarm.enemy[2].speed * changeInTime*slowerSpeed)
      swarm.enemy[2].buzzard2.x = swarm.enemy[2].buzzard2.x + (swarm.enemy[2].speed * changeInTime*slowerSpeed)
      
      swarm.enemy[2].buzzard3.x = swarm.enemy[2].buzzard3.x - (swarm.enemy[2].speed * changeInTime*slowerSpeed)
      swarm.enemy[2].buzzard4.x = swarm.enemy[2].buzzard4.x - (swarm.enemy[2].speed * changeInTime*slowerSpeed)
     else
     -- The "x" component, yet at normal speed.
      swarm.enemy[2].buzzard1.x = swarm.enemy[2].buzzard1.x + (swarm.enemy[2].speed * changeInTime)
      swarm.enemy[2].buzzard2.x = swarm.enemy[2].buzzard2.x + (swarm.enemy[2].speed * changeInTime)
      swarm.enemy[2].buzzard3.x = swarm.enemy[2].buzzard3.x - (swarm.enemy[2].speed * changeInTime)
      swarm.enemy[2].buzzard4.x = swarm.enemy[2].buzzard4.x - (swarm.enemy[2].speed * changeInTime)

     -- Arc the enemies so it looks like they're moving around the
     -- player.  Aren't mathematical functions just delightful?
      swarm.enemy[2].buzzard1.y = buzzard_1.y + (buzzard_1.y*(105/58)*changeInTime)
      swarm.enemy[2].buzzard2.y = buzzard_2.y + (buzzard_2.y*(105/58)*changeInTime)
      swarm.enemy[2].buzzard3.y = buzzard_3.y + (buzzard_3.y*(105/58)*changeInTime)
      swarm.enemy[2].buzzard4.y = buzzard_4.y + (buzzard_4.y*(105/58)*changeInTime)
     end
	
     -- Move the circle of each enemy for collision detecion.
     collision.moveBoundary(swarm.enemy[2].buzzard1.circle, swarm.enemy[2].buzzard1.x+32, swarm.enemy[2].buzzard1.y+32)
     collision.moveBoundary(swarm.enemy[2].buzzard2.circle, swarm.enemy[2].buzzard2.x+32, swarm.enemy[2].buzzard2.y+32)
     collision.moveBoundary(swarm.enemy[2].buzzard3.circle, swarm.enemy[2].buzzard3.x+32, swarm.enemy[2].buzzard3.y+32)
     collision.moveBoundary(swarm.enemy[2].buzzard4.circle, swarm.enemy[2].buzzard4.x+32, swarm.enemy[2].buzzard4.y+32)
    end,
    -- Actually draw them on the screen.
  moveBuzzards = function()
--     local tempList = {buzzardImage1, buzzardImage2, buzzardImage3, buzzardImage4}
    --  and not swarm.isOffScreen(swarm.enemy[2].buzzard1, buzzardImage1)
    if not swarm.enemy[2].buzzard1.circle.hasBeenHit then
      gf.draw(buzzardImage1, swarm.enemy[2].buzzard1.x, swarm.enemy[2].buzzard1.y)
    end
    --  and not swarm.isOffScreen(swarm.enemy[2].buzzard2, buzzardImage2)
    if not swarm.enemy[2].buzzard2.circle.hasBeenHit then
      gf.draw(buzzardImage2, swarm.enemy[2].buzzard2.x, swarm.enemy[2].buzzard2.y)
    end
    --  and not swarm.isOffScreen(swarm.enemy[2].buzzard3, buzzardImage3) 
    if not swarm.enemy[2].buzzard3.circle.hasBeenHit then
      gf.draw(buzzardImage3, swarm.enemy[2].buzzard3.x, swarm.enemy[2].buzzard3.y)
    end
    
    --  and not swarm.isOffScreen(swarm.enemy[2].buzzard4, buzzardImage4)
    if not swarm.enemy[2].buzzard4.circle.hasBeenHit then
      gf.draw(buzzardImage4, swarm.enemy[2].buzzard4.x, swarm.enemy[2].buzzard4.y)
    end
  end
  }

  -- Enemy table #3
  swarm.enemy[3] = { speed = 102, radius = 48, timeshot = 0, timeBetween = 0.4,
  readyToMove = nil, image = "images/smile.png",
  buzzard1 = { x = 40, y = -44, hasBeenHit = false, circle = nil },
  buzzard2 = { x = SCREENWIDTH-100, y = -44, hasBeenHit = false, circle = nil },
  -- Calculate the movements.
  zipBy = function(changeInTime)
   -- Calculate them to move slowly.
   if not swarm.enemy[3].buzzard1.circle.hasBeenHit then
      swarm.enemy[3].buzzard1.y = swarm.enemy[3].buzzard1.y + (swarm.enemy[3].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[3].buzzard1.circle, swarm.enemy[3].buzzard1.x+48, swarm.enemy[3].buzzard1.y+48)
   end
   if not swarm.enemy[3].buzzard2.circle.hasBeenHit then
      swarm.enemy[3].buzzard2.y = swarm.enemy[3].buzzard2.y + (swarm.enemy[3].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[3].buzzard2.circle, swarm.enemy[3].buzzard2.x+48, swarm.enemy[3].buzzard2.y+48)
   end
  end,
  -- Move them in slowly.
  moveBuzzards = function()
   if not swarm.enemy[3].buzzard1.circle.hasBeenHit then
     gf.draw(smileGuy1, swarm.enemy[3].buzzard1.x, swarm.enemy[3].buzzard1.y)
   end
   
   if not swarm.enemy[3].buzzard2.circle.hasBeenHit then
     gf.draw(smileGuy1, swarm.enemy[3].buzzard2.x, swarm.enemy[3].buzzard2.y)
   end
  end
  }

  -- Enemy table #4.
    -- There are a large number of enemies in this one.
  swarm.enemy[4] = {
   speed = 175, radius = 32, timeshot = 0, timeBetween = 0.4,
   image = "images/centurionrank.png", readyToMove = nil,
   -- First half
   buzzard1 = {x = -158, y = 158, hasBeenHit = false, circle = nil },
   buzzard2 = {x = -94, y = 94, hasBeenHit = false, circle = nil },
   buzzard3 = {x = -30, y = 30, hasBeenHit = false, circle = nil },
   buzzard4 = {x = 94, y = -94, hasBeenHit = false, circle = nil },
   buzzard5 = {x = 158, y = -158, hasBeenHit = false, circle = nil },
   -- Second half
   buzzard6 = {x = SCREENWIDTH+158, y = 158, hasBeenHit = false, circle = nil },
   buzzard7 = {x = SCREENWIDTH+94, y = 94, hasBeenHit = false, circle = nil },
   buzzard8 = {x = SCREENWIDTH+30, y = 30, hasBeenHit = false, circle = nil },
   buzzard9 = {x = SCREENWIDTH-94, y = -94, hasBeenHit = false, circle = nil },
   buzzard10 = {x = SCREENWIDTH-158, y = -158, hasBeenHit = false, circle = nil },
   shootingStars = function(changeInTime)
    local slope = 1.5
    -- Left flank
    if not swarm.enemy[4].buzzard1.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard1.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard1.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard1.circle, swarm.enemy[4].buzzard1.x, swarm.enemy[4].buzzard1.y)
    end
    if not swarm.enemy[4].buzzard2.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard2.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard2.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard2.circle, swarm.enemy[4].buzzard2.x, swarm.enemy[4].buzzard2.y)
    end
    if not swarm.enemy[4].buzzard3.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard3.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard3.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard3.circle, swarm.enemy[4].buzzard3.x, swarm.enemy[4].buzzard3.y)
    end
    if not swarm.enemy[4].buzzard4.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard4.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard4.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard4.circle, swarm.enemy[4].buzzard4.x, swarm.enemy[4].buzzard4.y)
    end
    if not swarm.enemy[4].buzzard5.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard5.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard5.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard5.circle, swarm.enemy[4].buzzard5.x, swarm.enemy[4].buzzard5.y)
    end
    if not swarm.enemy[4].buzzard6.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard6.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard6.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard6.circle, swarm.enemy[4].buzzard6.x, swarm.enemy[4].buzzard6.y)
    end
    if not swarm.enemy[4].buzzard7.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard7.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard7.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard7.circle, swarm.enemy[4].buzzard7.x, swarm.enemy[4].buzzard7.y)
    end
    if not swarm.enemy[4].buzzard8.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard8.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard8.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard8.circle, swarm.enemy[4].buzzard8.x, swarm.enemy[4].buzzard8.y)
    end
    if not swarm.enemy[4].buzzard9.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard9.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard9.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard9.circle, swarm.enemy[4].buzzard9.x, swarm.enemy[4].buzzard9.y)
    end
    if not swarm.enemy[4].buzzard10.circle.hasBeenHit then
      swarm.enemy[4].buzzard1.x = swarm.enemy[4].buzzard10.x + (slope * swarm.enemy[4].speed * changeInTime)
      swarm.enemy[4].buzzard1.y = swarm.enemy[4].buzzard10.y + (slope * swarm.enemy[4].speed * changeInTime)
      collision.moveBoundary(swarm.enemy[4].buzzard10.circle, swarm.enemy[4].buzzard10.x, swarm.enemy[4].buzzard10.y)
    end
--     for i=1,5 do
--       swarm.enemy[4]["buzzard"..i].x = swarm.enemy[4]["buzzard"..i].x + (slope * swarm.enemy[4].speed * changeInTime)
--       swarm.enemy[4]["buzzard"..i].y = swarm.enemy[4]["buzzard"..i].y + (slope * swarm.enemy[4].speed * changeInTime)
      -- You may want to add 32 to the components to center the boundaries.
--       collision.moveBoundary(swarm.enemy[4]["buzzard"..i].circle, swarm.enemy[4]["buzzard"..i].x, swarm.enemy[4]["buzzard"..i].y)
--     end
    -- Right flank
--     for i=6,10 do
--       swarm.enemy[4]["buzzard"..i].x = swarm.enemy[4]["buzzard"..i].x - (slope * swarm.enemy[4].speed * changeInTime) - 4 -- The 4 makes it better.
--       swarm.enemy[4]["buzzard"..i].y = swarm.enemy[4]["buzzard"..i].y + (slope * swarm.enemy[4].speed * changeInTime)
--       collision.moveBoundary(swarm.enemy[4]["buzzard"..i].circle, swarm.enemy[4]["buzzard"..i].x, swarm.enemy[4]["buzzard"..i].y)
--     end
   end,
   moveBuzzards = function()
     for i=1,10 do
       if not swarm.enemy[4]["buzzard"..i].circle.hasBeenHit then
	 gf.draw(tinyShip, swarm.enemy[4]["buzzard"..i].x, swarm.enemy[4]["buzzard"..i].y)
       end
     end
   end
  }

  -- Final boss
  swarm.enemy[5] = { -- Pretty sparse now.
    speed = nil, radius = nil,
    buzzard1 = { timeshot = nil, x = nil, y = nil, hasBeenHit = false, image = "images/bbeg.png" }
   }
  
  -- Animate enemy movement in the love.update() step.
  function swarm.move_enemies(changeInTime)
  -- Control the movement of the enemies.
   if swarm.enemy[1].readyToMove then
     swarm.enemy[1].zigzag(changeInTime)
   end

   if swarm.enemy[2].readyToMove then
     swarm.enemy[2].pincerAttack(changeInTime)
   end

   if swarm.enemy[3].readyToMove then
     swarm.enemy[3].zipBy(changeInTime)
   end

   if swarm.enemy[4].readyToMove then
     swarm.enemy[4].shootingStars(changeInTime)
   end
  end

  -- Animate enemy bullets.
  function swarm.shoot_enemy_weapons(changeInTime)
    --   <-- For the below argument.
   if not swarm.enemy[1].buzzard1.circle.hasBeenHit and not swarm.isOffScreen(swarm.enemy[1].buzzard1, enemy_image) then
     swarm.enemy[1].fireMainGun(swarm.enemy[1].buzzard1.x,swarm.enemy[1].buzzard1.y, changeInTime)

   -- The bullets for the next wave.
   elseif not swarm.isOffScreen(swarm.enemy[2].buzzard1, buzzardImage1) or not swarm.isOffScreen(swarm.enemy[2].buzzard2, buzzardImage2) or not swarm.isOffScreen(swarm.enemy[2].buzzard3, buzzardImage3) or not swarm.isOffScreen(swarm.enemy[2].buzzard4, buzzardImage4) then
     -- Slow down the shots.
    swarm.enemy[2].timeshot = swarm.enemy[2].timeshot - changeInTime
    if swarm.enemy[2].timeshot <= 0 then
     -- Add or subtract the radius of the enemy picture so that the
     -- bullets are centered when fired.
      armory.enemyWeapon[2].fireWeapon(swarm.enemy[2].buzzard1.x-swarm.enemy[2].radius, swarm.enemy[2].buzzard1.y+swarm.enemy[2].radius, 300, 90, "hamster")
      armory.enemyWeapon[2].fireWeapon(swarm.enemy[2].buzzard2.x-swarm.enemy[2].radius, swarm.enemy[2].buzzard2.y+swarm.enemy[2].radius, 300, 90, "hamster")
      armory.enemyWeapon[2].fireWeapon(swarm.enemy[2].buzzard3.x-swarm.enemy[2].radius, swarm.enemy[2].buzzard3.y+swarm.enemy[2].radius, 300, 90, "hamster")
      armory.enemyWeapon[2].fireWeapon(swarm.enemy[2].buzzard4.x-swarm.enemy[2].radius, swarm.enemy[2].buzzard4.y+swarm.enemy[2].radius, 300, 90, "hamster")
      swarm.enemy[2].timeshot = swarm.enemy[2].timeBetween
    end
   elseif not swarm.isOffScreen(swarm.enemy[3].buzzard1, smileGuy1) or not swarm.isOffScreen(swarm.enemy[3].buzzard2, smileGuy2) then
     -- Somehow, fire off bullets here.
   end
  end

  -- Actually draw them in the love.draw() step.
  function swarm.draw_enemies()
  -- ## First wave ##
   if swarm.enemy[1].readyToMove then
     swarm.enemy[1].moveBuzzard()
   end

  -- ## Second Wave ##
    if swarm.enemy[2].readyToMove then
     swarm.enemy[2].moveBuzzards()
    end

     -- ## Third wave ##
   if swarm.enemy[3].readyToMove then
     swarm.enemy[3].moveBuzzards()
   end

     -- ## Fourth wave ##
   if swarm.enemy[4].readyToMove then
     swarm.enemy[4].moveBuzzards()
   end
  end

  -- Set the images for the draw_enemies()
  -- function.
  function swarm.setImages(imageTable)
    tableOfImages = imageTable
  end

  -- Function for loading enemy images in the love.load() step.
  function swarm.loadEnemyImages()
   -- First wave of enemies.
   enemy_image = gf.newImage(swarm.enemy[1].image)
   -- Second wave of enemies.
   buzzardImage1 = gf.newImage(swarm.enemy[2].image)
   buzzardImage2 = gf.newImage(swarm.enemy[2].image)
   buzzardImage3 = gf.newImage(swarm.enemy[2].image)
   buzzardImage4 = gf.newImage(swarm.enemy[2].image)
   -- Third wave of enemies.
   smileGuy1 = gf.newImage(swarm.enemy[3].image)
   smileGuy2 = gf.newImage(swarm.enemy[3].image)
   -- Fourth wave of enemies.
   tinyShip = gf.newImage(swarm.enemy[4].image)
  end

  -- Return the "swarm" class.
  return swarm
end