--[[ This program is freeware; you may give it away and/or change it
under the GNU General Public License.  However, there is no
warrenty; not even an implied one.  When I say warrenty, you
say, "None!"  Also of note is that only the source code is
licensed under the GNU General Public License.  The art cannot
be edited, changed, or included with other programs.

The other software that this program uses is under a copyright
from the people whom wrote it, mostly being a variation on the
zlib license or BSD license.
For more information on the license, check the COPYING file
included with this program, or look on the GNU website,
www.gnu.org ]]

--[[
  What is contained in this file:
  * Stats for the hero.
  * Code for firing off hero's gun.
]]

Hero = {}

local SCREENWIDTH = love.graphics.getWidth()
local SCREENHEIGHT = love.graphics.getHeight()

function Hero.new()
 -- The table to hold all of this data, functions, and stuff.
 local ship = {}

 -- Set the armory object so as to register
 -- bullets for animating and collision detection.
 local armory = {}
 function ship.setArmory(weapon)
   armory = weapon
 end

 -- Certain values taken from the game SHMUP.
-- Hamster table.
 -- x = 356, y = 500
 ship.hamster = {x = SCREENWIDTH/2-35, y = SCREENHEIGHT+50, speed = 300, image = "images/hero.png", timeshot = 0, timeBetween = 0.2, radius = 32, hasBeenHit = false, lifePoints = 10, invincibilityLimit = 0}

 function ship.fireOffHeroWeapon()
     -- Use space bar or the left mouse button for firing.
      if love.keyboard.isDown(" ") or love.mouse.isDown("l") then
     -- Slow down the shots so that they don't
     -- look like one continuous line.
     if ship.hamster.timeshot <= 0 then
      -- Check which weapon is in use.
      if armory.weapon[1].inUse then
        armory.weapon[1].fireWeapon(ship.hamster.x, ship.hamster.y)
        -- Play a sound when the correct button is pressed (in this case,
        -- the space bar or the left mouse button).
        TEsound.play("sounds/ship_shooting_sound_2.ogg")
        ship.hamster.timeshot = ship.hamster.timeBetween  -- This has to be put here because...
      elseif armory.weapon[2].inUse then
	-- Slow down this rate of fire even more.
	armory.weapon[2].fireWeapon(ship.hamster.x, ship.hamster.y)
	if armory.weapon[2].bulletsFiredOff == 4 then
	  ship.hamster.timeshot = ship.hamster.timeshot - 0.2
	  armory.weapon[2].bulletsFiredOff = 0
	elseif armory.weapon[2].bulletsFiredOff < 4 then
	  armory.weapon[2].bulletsFiredOff = armory.weapon[2].bulletsFiredOff + 1
	end
        ship.hamster.timeshot = ship.hamster.timeBetween -- ...this value seems to screw up the pea-shooter (but not the spread gun).
      end
     end
   end
 end

 function ship.checkHeroLifeTotal(changeInTime)
    -- This is for the hero's life total.  Make sure that when the player
   -- is hit that it gives him a period of immunity.
   if ship.hamster.hasBeenHit then
     if ship.hamster.invincibilityLimit >= 0.75 then
       ship.hamster.hasBeenHit = false
       ship.hamster.lifePoints = ship.hamster.lifePoints - 1
       ship.hamster.invincibilityLimit = 0
     else
       ship.hamster.invincibilityLimit = ship.hamster.invincibilityLimit + changeInTime
     end
   else
     ship.hamster.invincibilityLimit = ship.hamster.invincibilityLimit + changeInTime
   end
 end
 
 return ship
end