--[[ This program is freeware; you may give it away and/or change it
under the GNU General Public License.  However, there is no
warrenty; not even an implied one.  When I say warrenty, you
say, "None!"  Also of note is that only the source code is
licensed under the GNU General Public License.  The art cannot
be edited, changed, or included with other programs.

The other software that this program uses is under a copyright
from the people whom wrote it, mostly being a variation on the
zlib license or BSD license.
For more information on the license, check the COPYING file
included with this program, or look on the GNU website,
www.gnu.org ]]

--[[
  What is contained in this file:
  * The code for the main Love steps.
  * The functions which call the actual code which does something.
]]

-- Include some outside code.
lhc = require "libs/hardoncollider"
require "libs/movement.lua"
require "libs/arsenal.lua"
require "libs/background.lua"
require "libs/enemies.lua"
require "libs/collision.lua"
require "libs/menus.lua"
require "libs/hero.lua"
require "libs/TEsound.lua"
require "libs/fader.lua"

  -- Get some code from other files.
  background = Background.new()
  armory = Weapons.new()
  horde = Enemies.new()
  movement = Movement.new()
  menus = Menus.new()
  hero = Hero.new()
  collision = Collision.new(armory.bullet, hero.hamster, horde)
  fader = fader:new()

  -- Some objects.
  kb = love.keyboard
  graf = love.graphics
  __squareRoot = math.sqrt
  __cosine = math.cos
  __sine = math.sin
  timer = love.timer

  -- Some constants.
  SCREENWIDTH = graf.getWidth()
  SCREENHEIGHT = graf.getHeight()

  -- How much time has gone by?
  totalTimeElapsed = 0
  -- dt = math.min(dt, 0.5) -- Let's keep this at a good pace (thank you, Thomas Holden).
  
-- ### The load step ###
function love.load()
   success = graf.setMode(800, 600, false, true, 0) -- Sets the resolution to 800x600, makes it windowed, turns on vsysnc, and uses no FSAA buffers.

   -- Load many images.
   hamster_image = graf.newImage(hero.hamster.image)

   -- Load enemy images.
   horde.loadEnemyImages()

   -- Background image.
   image2 = graf.newImage(background.image)

   -- For font stuff.
   graf.setFont(17)

   -- Give the weapons information object over to the
   -- class for the hero.
   hero.setArmory(armory)
   
   -- Load the collider.  By the way, should the cell size be
   -- bigger than 100?
   collision.startCollider()
   -- Load the enemy shapes.
   collision.loadEnemyShapes(horde)  
   -- Add the enemy shapes to the proper group.
   collision.addEnemiesToCollisionGroup(horde)
   -- Update the collider.
--    collision.updateCollider(30)
end

function start_colliding(changeInTime, thing1, thing2)
    if (thing1._groups.enemy and thing2._groups.bullets) or (thing1._groups.bullets and thing2._groups.enemy) then
      local enemy = thing1._groups.enemy and thing1 or thing2  -- Got to love ternary operations.
      if not enemy.hasBeenHit then
	enemy.hasBeenHit = true
        print("Enemy has been hit.")
      end
    end
  end

-- ### The update step ###
function love.update(dt)

  -- This is for creating a smoother frame rate.  Suggestion from the
  -- user vrid from the LOVE forums.
  --local step = 0.01

  -- Portion for menu.
 if not menus.letsGetThePartyStarted then
    menus.putInMenuSelections() -- Set up the menus.
    function love.keypressed(key, unicode)
     menus.keypressed(key, unicode)  -- Make sure that the player can move and select menu items.
    end

    -- The normal portion of the game
 elseif menus.letsGetThePartyStarted then

  -- Start the clock.
  totalTimeElapsed = totalTimeElapsed + dt

  -- Set the elapsed time for the draw_enemies() function.
  horde.setElapsedTime(totalTimeElapsed)

  -- Save all of the enemy images to a table
  -- so that they can be used in the various
  -- drawing functions.
  enemyImageTable = {enemy_image, buzzardImage1, buzzardImage2, buzzardImage3, buzzardImage4, smileGuy1, smileGuy2, tinyShips}

  -- Give over table of images to horde
  -- so as to draw enemies.
  horde.setImages(enemyImageTable)
  
  -- Fade-in effect (based upon Thomas Holden's code).
  if not fader.finished then
    fader:update(dt)
  end
  
-- ## Total movement for hero.
  if not movement.hasFinishedMoving then
    -- Big opening.
    movement.bigOpening(hero.hamster, 300, dt)
  elseif movement.hasFinishedMoving then
    -- Hero movement on the x-axis.
    hero.hamster.x = movement.moveOnXaxis(hero.hamster.x, hero.hamster.speed, dt)

    -- Hero movement on the y-axis.
    hero.hamster.y = movement.moveOnYaxis(hero.hamster.y,hero.hamster.speed, dt)
 end


   -- Press escape to...
    function love.keypressed(k)
       if k == 'escape' then
	 if love.audio then love.audio.stop() end -- Stop any sounds running.
         love.event.push('q') -- ...quit the game
       end
  -- Little debugging code for switching weapons.
--       elseif k == "x" then
--        if armory.weapon[2].inUse == false then
-- 	armory.weapon[1].inUse = false
-- 	weapon[2].inUse = true
--        elseif armory.weapon[2].inUse == true then
-- 	armory.weapon[2].inUse = false
-- 	armory.weapon[1].inUse = true
--        end
--       end
     end

     -- Update the measurements for the background.
     background:update(dt)

-- ## Enough movement code.

  -- More on collision.
     -- See if the enemy bullets hit the hero.
--      collision.checkForEnemyBulletCollision(armory.bullet, hero.hamster)
     -- See if the hero's bullets hit the enemy.
--      collision.checkForHeroBulletCollision()
     -- See if the enemies touched the hero.
--      collision.checkForEnemyCollision(hero,horde)
     
   -- This is for the hero's life total.  Make sure that when the player
   -- is hit that it gives him a period of immunity.
   if hero.hamster.hasBeenHit then
     if hero.hamster.invincibilityLimit >= 0.75 then
       hero.hamster.hasBeenHit = false
       hero.hamster.lifePoints = hero.hamster.lifePoints - 1
       hero.hamster.invincibilityLimit = 0
     else
       hero.hamster.invincibilityLimit = hero.hamster.invincibilityLimit + dt
     end
   else
     hero.hamster.invincibilityLimit = hero.hamster.invincibilityLimit + dt
   end
     -- Check the player's life total.
--      hero.checkHeroLifeTotal(dt)

   -- Check to see when the enemies should be sent out.
   horde.checkTimePast()
   
   -- Animate enemy movement.
    horde.move_enemies(dt)

   -- Animate enemy bullets.
    horde.shoot_enemy_weapons(dt)

   -- Taken from the game SHMUP.
   hero.hamster.timeshot = hero.hamster.timeshot - dt
   hero.fireOffHeroWeapon()

   -- Animate bullets
   armory.animate_bullets(dt)

   -- Remove spent shell casings.
   armory.cleanUpBullets()
   
   -- Update collider.
   lhc.update(dt)
  end
end

  -- ### The love.draw() step ###
function love.draw(dt)
  -- Menu portion.
  if not menus.letsGetThePartyStarted then
    menus.draw()

  elseif menus.letsGetThePartyStarted then  
  
  -- Draw 'em.
    -- First the background.
   background.draw_waves(image2)
   
   -- Fade-in code
   fader:draw()
   
    -- Then the hero.
   graf.draw(hamster_image, hero.hamster.x, hero.hamster.y)

    -- Next the enemies.
   horde.draw_enemies()

    -- And finally the bullets.
   armory.draw_bullets()

   -- Clean up the sound, apparently.
   TEsound.cleanup()

--    graf.print("Total elapsed time is "..totalTimeElapsed..".", 8, 500)
--    graf.print("The hero's life total is "..hero.hamster.lifePoints..".", 8, 517)
  end
end